import 'antd/dist/antd.css';
import React, { useContext, useEffect, useState } from 'react';

import { useUnauthRedirect } from '../../../hooks';
import { useRouter } from 'next/router';
import { Button, Input, Space, Table, Tag, Typography } from 'antd';
import { faFileLines } from '@fortawesome/free-solid-svg-icons';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { LoggedUserContext } from '../../../contexts/LoggedUserContext';
import { MainLayout } from '../../../layouts/MainLayout';
import { userController } from '../../../controllers';
import { AnnotationListInfo, EState } from '../../../api';
import {
    getAnnotationStateString,
    getAnnotationStateColor,
} from '../../../utils/strings';
import {
    CheckCircleOutlined,
    ClockCircleOutlined,
    SyncOutlined,
} from '@ant-design/icons';
import { getColumnSearchProps, getLocaleProps } from '../../../utils/tableUtils';

function UserDocumentPage() {
    const redirecting = useUnauthRedirect('/login');
    const { logout, role } = useContext(LoggedUserContext);
    const [documents, setDocuments] = useState<AnnotationListInfo[]>([]);
    const router = useRouter();

    useEffect(() => {
        async function fetchData() {
            let docs = (await userController.userAnnotationsGet()).data.annotations;
            if (!docs) {
                setDocuments([]);
            } else {
                setDocuments(docs);
            }
        }

        if (!redirecting /* && role === 'ANNOTATOR'*/) {
            // admin can also fetch data if he likes so
            fetchData();
        }
    }, [logout, redirecting, role, router]);

    const columns = [
        {
            title: 'Název dokumentu',
            dataIndex: 'documentName',
            key: 'documentName',
            ...getColumnSearchProps('documentName', 'název'),
            // @ts-ignore
            sorter: (a, b) => a.documentName.localeCompare(b.documentName),
        },
        {
            title: 'Stav anotace',
            key: 'state',
            dataIndex: 'state',
            render: (state: EState) => {
                const color = getAnnotationStateColor(state);
                const label = getAnnotationStateString(state);
                let icon = <CheckCircleOutlined />;
                if (state === 'NEW') {
                    icon = <ClockCircleOutlined />;
                }
                if (state === 'IN_PROGRESS') {
                    icon = <SyncOutlined />;
                }

                return (
                    <Tag icon={icon} color={color} key={label}>
                        {label.toUpperCase()}
                    </Tag>
                );
            },
            filters: [
                {
                    text: 'Nový',
                    value: 'NEW',
                },
                {
                    text: 'Rozpracováno',
                    value: 'IN_PROGRESS',
                },
                {
                    text: 'Hotovo',
                    value: 'DONE',
                },
            ],
            // @ts-ignore
            onFilter: (value, record) => record.state.indexOf(value) === 0,
            // @ts-ignore
            sorter: (a, b) => a.state.length - b.state.length,
            sortDirections: ['descend', 'ascend'],
        },
        {
            title: '',
            key: 'action',
            dataIndex: 'annotationId',
            render: (anotationId: number) => (
                <Button
                    key={anotationId}
                    type={'primary'}
                    onClick={() =>
                        router.push({
                            pathname: '/annotation/[annotationId]',
                            query: { annotationId: anotationId, final: false },
                        })
                    }
                >
                    Anotovat
                </Button>
            ),
        },
    ];

    return redirecting /*|| role !== 'ANNOTATOR' */ ? null : (
        <MainLayout>
            <Typography.Title level={2}>
                <FontAwesomeIcon icon={faFileLines} /> Dokumenty
            </Typography.Title>
            <Table
                locale={{ ...getLocaleProps() }}
                // @ts-ignore
                columns={columns}
                // @ts-ignore
                dataSource={documents}
                size="small"
                scroll={{ y: 500 }}
            />
        </MainLayout>
    );
}

export default UserDocumentPage;
